
#include "stdio.h"
#include "stdlib.h"
#include "listP.h"

static LIST fe_nodes = NULL;
static LIST fe_elements = NULL;

typedef struct node_t {
  double x, y, z;
  int    n;
} node_t;

typedef struct elem_t {
  int n[10], layer;
} elem_t;

#define EPS 0.0001

#define CMP_XYZ(X1, Y1, Z1, X2, Y2, Z2) (   (fabs((X2)-(X1)) < EPS )  \
                                         && (fabs((Y2)-(Y1)) < EPS )  \
                                         && (fabs((Z2)-(Z1)) < EPS ))

static int
add_node (double x, double y, double z)
{
  int n = 0;
  node_t *nr = NULL;

  nr = (node_t *)get_list_next(fe_nodes, nr);
  while (nr != NULL) {
    n++;
    if (CMP_XYZ(x, y, z, nr->x, nr->y, nr->z))
      return nr->n;
    nr = (node_t *)get_list_next(fe_nodes, nr);
  }
  nr = (node_t *)make_node(sizeof(node_t));
  nr->x = x, nr->y = y, nr->z = z, nr->n = ++n;
  add_to_tail(fe_nodes, nr);
  return n;
}

static int 
add_elem (int n[10], int layer)
{
  elem_t *el;
  int i;
  
  el = (elem_t *)make_node(sizeof(elem_t));
  for (i = 0; i < 10; i++) {
    el->n[i] = n[i];
  }
  el->layer = layer;
  add_to_tail(fe_elements, el);
  return 1;
}

static int 
dump_elems (void)
{
  elem_t *el = NULL;
  int i;

  printf ("! >>>>\n");
  printf ("! %d tet10 cells\n", list_length(fe_elements));
  el = (elem_t *) get_list_next (fe_elements, el);
  while (el != NULL) {
    for (i = 0; i < 10; i++) {
      printf("%d ", el->n[i]);
    }
    printf(" ! group %d\n", el->layer);
    el = (elem_t *) get_list_next (fe_elements, el);
  }
  printf ("! <<<<\n");
  return 1;
}


static int
dump_nodes(void)
{
  node_t *nr = NULL;

  printf ("! >>>>\n");
  printf ("! %d fen's\n", list_length(fe_nodes));
  nr = (node_t *)get_list_next(fe_nodes, nr);
  while (nr != NULL) {
    printf("%d %f %f %f\n", nr->n, nr->x, nr->y, nr->z);
    nr = (node_t *)get_list_next(fe_nodes, nr);
  }
  printf ("! <<<<\n");
  return 1;
}

int 
main(int argc, char **argv)
{
  FILE *stream;
#define LINELENGTH 2048
  char buffer[2048];
  double x[10], y[10], z[10];
  int layer;

  if (argc == 1) {
    fprintf(stderr, "No input file!?\n");
    exit(0);
  }
  
  if ((stream = fopen(argv[1], "r")) == NULL) {
    fprintf(stderr, "Failed open input file %s!?\n", argv[1]);
    exit(0);
  }

  fe_nodes = make_list();
  fe_elements = make_list();

  layer = 0;
  while (fgets (buffer, LINELENGTH-1, stream) != NULL) {
    if (strncmp (buffer, "GID", 3) == 0) {
      int timestamp, serial, status
        = sscanf(buffer,
                 "GID time stamp: %d, serial: %d, layer %d",
                 &timestamp, &serial, &layer);
      if (status != 3) {
        fprintf(stderr, "Failed read input file %s!?\n", argv[1]);
        fprintf(stderr, "Got: %s\n", buffer);
      }
    }
    if (strncmp (buffer, "TET10", 5) == 0) {
      int status
        = sscanf(buffer,
                 "TET10 [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf]",
                 &x[0], &y[0], &z[0],
                 &x[1], &y[1], &z[1],
                 &x[2], &y[2], &z[2],
                 &x[3], &y[3], &z[3],
                 &x[4], &y[4], &z[4],
                 &x[5], &y[5], &z[5],
                 &x[6], &y[6], &z[6],
                 &x[7], &y[7], &z[7],
                 &x[8], &y[8], &z[8],
                 &x[9], &y[9], &z[9]
                 );
      if (status != 30) {
        fprintf(stderr, "Failed read input file %s!?\n", argv[1]);
        fprintf(stderr, "Got: %s\n", buffer);
      }
      {
        int i, n[10];
        double J[3][3], detJ;
        for (i = 0; i < 10; i++) {
          n[i] = add_node (x[i], y[i], z[i]);
        }
        G3D_tet10_J (J, 0.25, 0.25, 0.25, 0.25,
                     x[0], y[0], z[0],
                     x[1], y[1], z[1],
                     x[2], y[2], z[2],
                     x[3], y[3], z[3],
                     x[4], y[4], z[4],
                     x[5], y[5], z[5],
                     x[6], y[6], z[6],
                     x[7], y[7], z[7],
                     x[8], y[8], z[8],
                     x[9], y[9], z[9]);
        detJ = G3D_tet10_detJ (J);
        if (detJ <= 0) {        /* try swapping 1<->2, 4<->6, 8<->9 */
          G3D_tet10_J (J, 0.25, 0.25, 0.25, 0.25,
                       x[0], y[0], z[0],
                       x[2], y[2], z[2],
                       x[1], y[1], z[1],
                       x[3], y[3], z[3],
                       x[6], y[6], z[6],
                       x[5], y[5], z[5],
                       x[4], y[4], z[4],
                       x[7], y[7], z[7],
                       x[9], y[9], z[9],
                       x[8], y[8], z[8]
                       );
          detJ = G3D_tet10_detJ (J);
          if (detJ <= 0) {
            fprintf (stderr, "Non-positive Jacobian tet\n");
            exit (1);
          }
          { int i; i = n[1]; n[1] = n[2]; n[2] = i; }
          { int i; i = n[4]; n[4] = n[6]; n[6] = i; }
          { int i; i = n[8]; n[8] = n[9]; n[9] = i; }
        }
        add_elem (n, layer);
      }
      layer = 0;
    }
    if (strncmp (buffer, "TETRA", 5) == 0) {
      int status
        = sscanf(buffer,
                 "TETRA [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf] [%lf,%lf,%lf]",
                 &x[0], &y[0], &z[0],
                 &x[1], &y[1], &z[1],
                 &x[2], &y[2], &z[2],
                 &x[3], &y[3], &z[3]
                 );
      if (status != 12) {
        fprintf(stderr, "Failed read input file %s!?\n", argv[1]);
        fprintf(stderr, "Got: %s\n", buffer);
      }
      {
        int i, n[10];
        double J[3][3], detJ;
        detJ = G3D_tet_volume (x[0], y[0], z[0],
                     x[1], y[1], z[1],
                     x[2], y[2], z[2],
                     x[3], y[3], z[3]);
        if (detJ <= 0) {        /* try swapping 1<->2 */
          { int i; i = n[1]; n[1] = n[2]; n[2] = i; }
        }
        /* Generate the other nodes */
        for (i = 0; i < 4; i++) {
          n[i] = add_node (x[i], y[i], z[i]);
        }
        n[4] = add_node ((x[0]+x[1])/2, (y[0]+y[1])/2, (z[0]+z[1])/2);
        n[5] = add_node ((x[2]+x[1])/2, (y[2]+y[1])/2, (z[2]+z[1])/2);
        n[6] = add_node ((x[2]+x[0])/2, (y[2]+y[0])/2, (z[2]+z[0])/2);
        n[7] = add_node ((x[3]+x[0])/2, (y[3]+y[0])/2, (z[3]+z[0])/2);
        n[8] = add_node ((x[3]+x[1])/2, (y[3]+y[1])/2, (z[3]+z[1])/2);
        n[9] = add_node ((x[3]+x[2])/2, (y[3]+y[2])/2, (z[3]+z[2])/2);
        /* Add elem */
        add_elem (n, layer);
      }
      layer = 0;
    }
  }
  
  dump_nodes ();
  dump_elems ();

  fclose(stream);
  return 1;
}
  









