#include "ioP.h"
#include "tokensP.h"
#include "dictP.h"
#include "getoptP.h"
#include "newP.h"
/*
  This program takes the data produced by VDT when the option
  -dump_data is given.  The `pt' lines become lines in the fenf.tmp
  file, the `bsf' lines become lines in the ebcf.tmp file (of the ntri type),
  and the definition of the tetrahedra ends up in the file connf.tmp.
 */

static dict_dictionary sd = NULL;
static int report_opts = FALSE;
static char *slist = NULL;

static void
make_sd (void)
{
  tokens_parser parser = tokens_new_parser ();
  sd = dict_create_dictionary ();
  if (tokens_parse_line (parser, slist)) {
    int i;
    for (i = 1; i <= tokens_total_of_tokens (parser); i++) {
      int sid = tokens_token_as_int (parser, i);
      dict_add_obj_int_name (sd, (dict_object) sid, sid);
    }
  }
  tokens_delete_parser (parser);
}
  

static int
usage (int argc, char **argv)
{
  char *option, *usage;

  fprintf (stderr, "usage: %s vdtfile + options below\n", argv[0]);
  
  if (ckit_opt_first_recognized (&option, &usage)) {
    fprintf (stderr, "%s %s\n",
             (option != NULL ? option : "?"),
             (usage  != NULL ? usage  : "?"));
  }
  while (ckit_opt_next_recognized (&option, &usage)) {
    fprintf (stderr, "%s %s\n",
             (option != NULL ? option : "?"),
             (usage  != NULL ? usage  : "?"));
  }
  exit (0);
  return argc;
}

static void 
process_options (int argc, char **argv)
{
  int help = FALSE;

#define DOSTRINGZ(s) #s
#define STRINGZ(s) DOSTRINGZ(s)
#define SET_BOOL(name)                                          \
  {                                                             \
    if (report_opts) { fprintf (stderr, "Set: %s\n", #name); }  \
    name = TRUE;                                                \
  }
#define SET_WITH_ARG(name, format)                                        \
  {                                                                       \
    if (report_opts) { fprintf (stderr, "Set: %s = %" #format "\n",       \
                                STRINGZ(name), name); }                   \
  }
  
  GET_OPT_BOOL (argc, argv, "-report_opts", SET_BOOL(report_opts),
                "prints options as they are being set");
  
  GET_OPT_BOOL (argc, argv, "-help", SET_BOOL(help),
                "prints usage for each option recognized");
  
  slist = NULL;
  GET_OPT_ARG (argc, argv, "-slist", char, slist,
               { SET_WITH_ARG (slist, s); },
               "list of surface id's on which to apply the EBC");
  if (slist) {
    const char *s = slist;
    NEWP_DUPL_STRING (slist, s, { fprintf(stderr, "No memory\n"); exit (1); });
    make_sd ();
  }

  if (help)
    usage (argc, argv);
}

static int
in_slist (int id)
{
  if ((slist == NULL) || (sd == NULL)) return TRUE;
  return (dict_find_obj_with_int_name (sd, id) == (dict_object) id);
}

int
main (int argc, char **argv)
{
  tokens_parser parser = tokens_new_parser ();
  if (!parser) return -1;
  if (argc < 2) {
    fprintf (stderr, "No vdt file name given?\n");
    return -2;
  }
  if (!tokens_open_file (parser, argv[1])) return -3;
  process_options (argc, argv);
  {
    const char *fenf = "fenf.tmp";
    const char *ebcf = "ebcf.tmp";
    const char *connf = "connf.tmp";
    FILE *fenfp, *ebcfp, *connfp;
    fenfp = fopen (fenf, "w");
    if (!fenfp) {
      fprintf (stderr, "Could not open %s\n", fenf);
      return -4;
    }
    ebcfp = fopen (ebcf, "w");
    if (!ebcfp) {
      fprintf (stderr, "Could not open %s\n", ebcf);
      return -4;
    }
    fprintf (ebcfp, "function \"0\"\n");
    connfp = fopen (connf, "w");
    if (!connfp) {
      fprintf (stderr, "Could not open %s\n", connf);
      return -4;
    }
    while (tokens_next_line (parser)) {
      if (tokens_type_of_token (parser, 1) == TOKENS_NUMBER) { /* tetrahedron */
        fprintf (connfp, "%d %d %d %d\n",
                 tokens_token_as_int (parser, 2),
                 tokens_token_as_int (parser, 3),
                 tokens_token_as_int (parser, 4), 
                 tokens_token_as_int (parser, 5)); 
      } else {
        const char *keyw = tokens_token_as_string (parser, 1);
        if        (tokens_token_and_keyword_equiv (keyw, "bsf")) {
          int sid = tokens_token_as_int (parser, 5);
          if (in_slist (sid)) {
            fprintf (ebcfp, "ntri %d %d %d dist %g ! surf %d\n",
                     tokens_token_as_int (parser, 2),
                     tokens_token_as_int (parser, 3),
                     tokens_token_as_int (parser, 4), 0.00001,
                     tokens_token_as_int (parser, 5));
          }
        } else if (tokens_token_and_keyword_equiv (keyw, "pt")) {
          fprintf (fenfp, "%d %g %g %g\n", tokens_token_as_int (parser, 2),
                   tokens_token_as_double(parser, 3), tokens_token_as_double(parser, 4), tokens_token_as_double(parser, 5));
        }
      }
    }
    fclose (fenfp);
    fclose (ebcfp);
    fclose (connfp);
  }
  tokens_delete_parser (parser);
  return 0;
}
